/*
 * @(#)CDWverify.cpp
 *
 * This file is part of webCDwriter - Network CD/DVD Writing.
 *
 * Copyright (C) 1999-2005 Jrg P. M. Haeger
 *
 * webCDwriter is free software. See CDWserver.cpp for details
 *
 * Jrg P. M. Haeger, 17.11.2002
 */

#include <sys/types.h>
#include <dirent.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

typedef unsigned long long uint64;

/**
 * A simple CDWverify simulation
 *
 * @version 20050826
 * @author Jrg P. M. Haeger
 */
class Verify {
public:
	static bool cancel;

private:
	int existing, missing;
	uint64 origSize;

public:
	int getExisting() { return existing; }
	int getMissing() { return missing; }

	int getOrigSizeMB() {
		return (int) (origSize / 1024 / 1024);
	}

	public: void scan(const char *orig) {
		existing = missing = 0;
		origSize = 0;
		scan2(orig);
	}

	private: void scan2(const char *orig) {
		struct stat statBuf;
		if (stat(orig, &statBuf) != 0)
			return;

		if (!S_ISDIR(statBuf.st_mode)) {
			origSize += statBuf.st_size;
			existing++;
		}

		DIR *dir = opendir(orig);
		if (dir == NULL)
			return;

		struct dirent *dirEntry;
		while ((dirEntry = readdir(dir)) != NULL) {
			if (strcmp(dirEntry->d_name, ".") == 0
					|| strcmp(dirEntry->d_name, "..") == 0)
				continue;

			char tmpOrig[strlen(orig) + 1
				+ strlen(dirEntry->d_name) + 1];
			sprintf(tmpOrig, "%s/%s",
				orig, dirEntry->d_name);

			scan2(tmpOrig);
			if (cancel)
				break;
		}

		closedir(dir);
	}
};

bool Verify::cancel = false;

void sigTermHandler(int sig) {
	Verify::cancel = true;
	printf("\nCancelled\n");
	fflush(stdout);
}

int main(int argc, char *args[]) {
	printf("CDWverify-dummy 20050826 (part of webCDwriter)\n");
	printf("\nAttention: This is just a simulation of CDWverify!\n\n");
	fflush(stdout);

	signal(SIGTERM, sigTermHandler);

	if (argc >= 2 && strstr(args[1], "-readISOHeader") != NULL)
		return 0;

	const char *orig = "/bin/";
	if (argc >= 3)
		orig = args[argc - 2];

	Verify verify;

	printf("Scanning the directories...");
	fflush(stdout);
	verify.scan(orig);
	printf(" %d of %d files found\n",
		verify.getExisting(),
		verify.getExisting() + verify.getMissing());
	printf("%d MB total\n", verify.getOrigSizeMB());
	fflush(stdout);

	int sizeMB = verify.getOrigSizeMB();
	if (sizeMB < 1)
		sizeMB = 1;
	int files = 0;
	for (int i = 1; i <= sizeMB && !Verify::cancel; i++) {
		files = i * verify.getExisting() / sizeMB;
		printf("\r%d MB, %d files, %d errors, %4.1fx (%4.0f KB/s)",
			i, files, 0, 6.0, 900.0);
		fflush(stdout);
		usleep(20 * 1000 * 1000 / sizeMB);
	}

	printf("\n%d files are identical, 0 files differ\n", files);

	return 0;
}
