#!/bin/sh

#
# tools/CDWserverPro-installer.sh
#
# Copyright (C) 2002-2007 Jrg P. M. Haeger
#

# Thanks to Ramon Casha for the "Self-extracting Archive HOWTO"
# (see http://linux.org.mt/article/selfextract)

CONFIGDIR=/etc/CDWserver
INITDIR=/etc/init.d
MD5="@MD5@"
PRODUCT=CDWserver
PRO=Pro
SBINDIR=/usr/sbin
STATIC=false
VERSION=@VERSION@

echo
echo "$PRODUCT $PRO $VERSION"
echo

KNOPPIX="`grep loop /proc/mounts | grep ' /KNOPPIX '`"
if [ "$KNOPPIX" ]; then
	if [ -e "/var/CDWserver/sbin/$PRODUCT" ]; then
		SBINDIR=/var/CDWserver/sbin
	fi
	unset SUDO_COMMAND
fi

if [ ! "$1" = "" ]; then
	SBINDIR="$1"
fi

DEST="$SBINDIR"

if [ ! -e "$DEST/$PRODUCT" ]; then
	echo "The server binary $DEST/$PRODUCT does not exist."
	echo "Make sure webCDwriter is installed!"
	echo
	echo "The default SBINDIR is \"$SBINDIR\"."
	echo "Retry"
	echo "   sh $0 SBINDIR"
	echo "with the SBINDIR you used in the configure script!"
	echo
	exit 1
fi

SKIP=`awk '/^__ARCHIVE_FOLLOWS__/ { print NR + 1; exit 0; }' $0`

echo -n "Verifying the checksum..."
if [ -x /sbin/md5 ]; then
	# FreeBSD
	MD5TOOL="/sbin/md5 -q"
else
	MD5TOOL=md5sum
fi
MD5test="`tail -n +$SKIP $0 | $MD5TOOL`"
if [ ! "$MD5test" = "$MD5" ]; then
	MD5test="$MD5test  -"
	if [ ! "$MD5test" = "$MD5" ]; then
		echo " does not match -> retry to download this installer!"
		echo
		exit 1
	fi
fi
echo " OK"

echo -n "Looking for bzip2... "
if ! which bzip2 2> /dev/null; then
	echo "not found"
	echo
	echo "Please install the tool \"bzip2\"!"
	echo
	exit 1
fi

if [ ! "`id -u`" = "0" ]; then
	echo
	echo "This installer will replace $DEST/$PRODUCT."
	echo "To do so run it as root!"
	echo
	exit 1
fi

more << EOF

This version of CDWserver is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

EOF

echo "Do you accept? (yes or no)"
read input
if [ ! "$input" = "yes" ]; then
	echo "Installation cancelled!"
	exit 1
fi

echo
echo "$PRODUCT $PRO $VERSION - starting installation... please wait"
echo

if [ "$STATIC" = "false" ]; then
	if [ ! -e /usr/lib/libcrypto.so.0 ] && [ ! -e /lib/libcrypto.so.0 ]; then
		if [ -e /lib/libcrypto.so.4 ]; then
			# Fedora Core release 1.91 (FC2)
			ln -s libcrypto.so.4 /lib/libcrypto.so.0
		elif [ -e /usr/lib/libcrypto.so.0.9.7 ]; then
			# Mandrake Linux release 10.0
			ln -s libcrypto.so.0.9.7 /usr/lib/libcrypto.so.0
		elif [ -e /usr/lib/libcrypto.so.0.9.8 ]; then
			# Ubuntu 6.06, 6.10, and 7.04
			ln -s libcrypto.so.0.9.8 /usr/lib/libcrypto.so.0
		elif [ -e /usr/lib/libcrypto.so ]; then
			# FC5
			ln -s libcrypto.so /usr/lib/libcrypto.so.0
		else
			echo "No /usr/lib/libcrypto.so.0"
			echo
			exit 1
		fi
	fi
	if [ ! -e /usr/lib/libssl.so.0 ] && [ ! -e /lib/libssl.so.0 ]; then
		if [ -e /lib/libssl.so.4 ]; then
			# Fedora Core release 1.91 (FC2)
			ln -s libssl.so.4 /lib/libssl.so.0
		elif [ -e /usr/lib/libssl.so.0.9.7 ]; then
			# Mandrake Linux release 10.0
			ln -s libssl.so.0.9.7 /usr/lib/libssl.so.0
		elif [ -e /usr/lib/libssl.so.0.9.8 ]; then
			# Ubuntu 6.06 LTS, 6.10, and 7.04
			ln -s libssl.so.0.9.8 /usr/lib/libssl.so.0
		elif [ -e /usr/lib/libssl.so ]; then
			# FC5
			ln -s libssl.so /usr/lib/libssl.so.0
		else
			echo "No /usr/lib/libssl.so.0"
			echo
			exit 1
		fi
	fi
fi

if [ -x /etc/rc.d/CDWserver ]; then
	# FreeBSD
	INITDIR=/etc/rc.d
fi

$INITDIR/CDWserver stop
if [ ! -e $DEST/$PRODUCT-GPL ]; then
	echo -n "Moving $DEST/$PRODUCT to $DEST/$PRODUCT-GPL..."
	mv $DEST/$PRODUCT $DEST/$PRODUCT-GPL
	echo " done"
fi
echo -n "Installing $DEST/$PRODUCT..."
sleep 3
tail -n +$SKIP $0 | bzip2 -cd - > $DEST/$PRODUCT
chmod +x $DEST/$PRODUCT
echo " done"
$INITDIR/CDWserver start

PORT="`grep '^portNo=' $CONFIGDIR/config | sed s/portNo=//g`"
if [ "$PORT" = "80" ]; then
	PORT_EXT=""
else
	if [ "$PORT" = "" ]; then
		PORT_EXT=":12411"
	else
		PORT_EXT=":$PORT"
	fi
fi

echo
echo "To do:"
echo
echo "Open your web browser and try"
echo
echo "   \"http://127.0.0.1$PORT_EXT/about.html\""
echo "or"
echo "   \"http://`hostname`$PORT_EXT/about.html\""
echo
echo "to check the version and license of your webCDwriter."
echo

exit 0

__ARCHIVE_FOLLOWS__
